#!/bin/bash
PATH=$PATH:/sbin

IWSS_BIN="/usr/iwss"
IWSS_DATA="/var/iwss"

ATS_ROOT_DIR=/usr/iwss/ats
ATS_ETC_DIR=${ATS_ROOT_DIR}/etc/trafficserver
ATS_BIN_DIR=${ATS_ROOT_DIR}/bin

ATS_CACHE_LOCATION=
ATS_CACHE_SIZE=

CACHE_CONF=$IWSS_DATA/iwssCache.ini
CACHE_SECTION=Cache
RECORD_SECTION=Record

INTSCAN_CONF=$IWSS_DATA/intscan.ini
HTTP_CONF=$IWSS_DATA/IWSSPIProtocolHttpProxy.pni

CACHE_CLEAR_MARK=/var/iwss/CACHE_CLEAR.MARK

CACHE_LOG=/var/iwss/log/cache_config.log

RWCONFIG=/usr/bin/rwconfig
INICONFIG=/usr/bin/iniconfig

CACHE_EXCEPTION_LIST=/var/iwss/cacheExceptionList.txt

dbgLog()
{
	echo -e "`date`: $@" >> $CACHE_LOG
}

fnIsProcessAlive()
{
	if [ -x /sbin/pidof ] ; then
		/sbin/pidof `basename $*` > /dev/null
	elif [ -x /bin/pidof ] ; then
		/bin/pidof `basename $*` > /dev/null
	else
		ps -ef | grep $* | grep -v grep > /dev/null
	fi
}

setAtsConfig()
{
	dbgLog setAtsConfig $1 $2 $3

	old_line=`cat $ATS_ETC_DIR/records.config| grep -v "#" |grep $1`

	if [ "$old_line" != "" ]; then
		new_line=`echo $old_line|awk -F' ' '{ print $1 " " $2 " " $3}'`
		new_line="$new_line $2"

		if [ "$old_line" = "$new_line" ]; then
			return
		fi

		sed -i "s/$old_line/$new_line/" $ATS_ETC_DIR/records.config
	else
		echo "CONFIG $1 $3 $2" >> $ATS_ETC_DIR/records.config
	fi
}

getATSStorage()
{
	if [ ! -f $ATS_ETC_DIR/storage.config ]; then
		ATS_CACHE_LOCATION=""
		ATS_CACHE_SIZE=""

		return
	fi

	line=`cat $ATS_ETC_DIR/storage.config|grep -v "#"`

	ATS_CACHE_LOCATION=`echo $line|cut -d' ' -f1`
	ATS_CACHE_SIZE=`echo $line|cut -d' ' -f2`

	dbgLog getATSStorage $ATS_CACHE_LOCATION $ATS_CACHE_SIZE
}

setATSStorage()
{
	if [ "$1" != "" -a "$2" != "" ]; then
		dbgLog setATSStorage $1 $2

		cat $ATS_ETC_DIR/storage.config | grep "#" > /tmp/a.txt
		cp -f /tmp/a.txt $ATS_ETC_DIR/storage.config
		rm -f /tmp/a.txt

		echo $1 $2 >> $ATS_ETC_DIR/storage.config
	fi
}

addATSException()
{
	line=$1

	f2=`echo "$line"|cut -d"@" -f2 -s`

	if [ "$f2" != "" ]; then
		f1=`echo "$f2"|cut -d"*" -f1 -s`

		if [ "$f1" != "" ]; then
			# by domain
			echo "dest_domain=$f1 action=never-cache" >> $ATS_ETC_DIR/cache.config
			echo "dest_host=$f1 action=never-cache" >> $ATS_ETC_DIR/cache.config
		fi

		return
	fi

	f2=`echo "$line"|cut -d"*" -f2 -s`

	if [ "$f2" != "" ]; then
		f1=$f2

		# by url key word
		echo "url_regex=.*$f1.* action=never-cache" >> $ATS_ETC_DIR/cache.config

		return
	fi

	f1=$line
	# by url
	echo "url_regex=.*$f1 action=never-cache" >> $ATS_ETC_DIR/cache.config

	return
}

fnStartATS()
{
	CACHE_ENABLED=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "enable")
	dbgLog fnStartATS "CACHE_ENABLED=$CACHE_ENABLED"

	if [ "$CACHE_ENABLED" != "yes" ]; then
		return
	fi

	if [ ! -f ${ATS_BIN_DIR}/trafficserver ]; then
		dbgLog fnStartATS "not found ${ATS_BIN_DIR}/trafficserver"

		return
	fi

	# ATS storage
	ATS_CACHE_LOCATION=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "cache_directory")
	ATS_CACHE_SIZE=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "cache_size")

	if [ "$ATS_CACHE_LOCATION" != "" ]; then
		mkdir -p $ATS_CACHE_LOCATION
		chmod -R 0777 $ATS_CACHE_LOCATION
	fi

	MEM_CACHE=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "memory_cache")

	dbgLog fnStartATS "ATS_CACHE_LOCATION=$ATS_CACHE_LOCATION ATS_CACHE_SIZE=$ATS_CACHE_SIZE MEM_CACHE=$MEM_CACHE"

	if [ "$ATS_CACHE_LOCATION" != "" -a "$ATS_CACHE_SIZE" != "" ]; then
		dbgLog check tmpfs $ATS_CACHE_LOCATION

		CHECK_TMPFS=`mount | grep -v grep | grep $ATS_CACHE_LOCATION | wc -l`
		if [ $CHECK_TMPFS -ne 0 ]; then
			if  fnIsProcessAlive "traffic_server" ; then
				dbgLog "ats is already running."

				${ATS_BIN_DIR}/trafficserver stop >/dev/null 2>&1

				killall -9 traffic_cop
				killall -9 traffic_manager
				killall -9 traffic_server
			fi

			dbgLog umount $ATS_CACHE_LOCATION

			umount $ATS_CACHE_LOCATION
		fi

		if [ "$MEM_CACHE" = "yes" ]; then
			dbgLog mount tmpfs $ATS_CACHE_LOCATION

			mount tmpfs $ATS_CACHE_LOCATION -t tmpfs -o size=$ATS_CACHE_SIZE
		fi
	fi

	setATSStorage $ATS_CACHE_LOCATION $ATS_CACHE_SIZE

	DPT_PORT=80

	# we also need to check if the upstream proxy was enabled, if so, add the upstream proxy
	# to ats's config file
	SelfProxy=$($INICONFIG -r -f $HTTP_CONF -s "http" -o "self_proxy")

	if [[ "$SelfProxy" = "no" ]]; then
		ProxyIP=$($INICONFIG -r -f $HTTP_CONF -s "http" -o "original_server")
		ProxyPort=$($INICONFIG -r -f $HTTP_CONF -s "http" -o "original_server_port")

		if [[ -n "$ProxyIP" && -n "$ProxyPort" ]]; then
			dbgLog "Enable the upstream proxy, proxyIP=$ProxyIP, proxyPort=$ProxyPort"

			cat $ATS_ETC_DIR/parent.config | grep "#" > /tmp/a.txt
			cp -f /tmp/a.txt $ATS_ETC_DIR/parent.config
			rm -f /tmp/a.txt

			line="dest_domain=.  parent=$ProxyIP:$ProxyPort  round_robin=false"
			echo $line >> $ATS_ETC_DIR/parent.config

			dbgLog parent.config is "$line"

			setAtsConfig proxy.config.http.parent_proxy_routing_enable 1 INT

			DPT_PORT=$ProxyPort
		else
			setAtsConfig proxy.config.http.parent_proxy_routing_enable 0 INT
		fi
	else
		dbgLog "There is no upstream proxy, clear ATS upstream proxy setting"

		cat $ATS_ETC_DIR/parent.config | grep "#" > /tmp/a.txt
		cp -f /tmp/a.txt $ATS_ETC_DIR/parent.config
		rm -f /tmp/a.txt

		setAtsConfig proxy.config.http.parent_proxy_routing_enable 0 INT
	fi

	# set ATS config
	setAtsConfig proxy.config.log.logging_enabled 0 INT
	setAtsConfig proxy.config.log.max_space_mb_for_logs 1024 INT
	setAtsConfig proxy.config.log.custom_logs_enabled 0 INT
	setAtsConfig proxy.config.log.squid_log_enabled 0 INT
	setAtsConfig proxy.config.log.common_log_enabled 0 INT
	setAtsConfig proxy.config.log.extended_log_enabled 0 INT
	setAtsConfig proxy.config.log.extended2_log_enabled 0 INT

	ATS_PORT=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "ats_listen_port")
	setAtsConfig proxy.config.http.server_ports $ATS_PORT STRING

	setAtsConfig proxy.config.http.cache.http 1 INT
	setAtsConfig proxy.config.http.enabled 1 INT

	setAtsConfig proxy.config.http.uncacheable_requests_bypass_parent 0 INT

	setAtsConfig proxy.config.http.insert_squid_x_forwarded_for 0 INT
	setAtsConfig proxy.config.http.anonymize_insert_client_ip 0 INT

	setAtsConfig proxy.config.reverse_proxy.enabled 0 INT
	setAtsConfig proxy.config.url_remap.remap_required 0 INT
	setAtsConfig proxy.config.http2.enabled 0 INT
	setAtsConfig proxy.config.socks.socks_needed 0 INT

	setAtsConfig proxy.config.cluster.cluster_port 5086 INT
	setAtsConfig proxy.config.cluster.rsport 5088 INT
	setAtsConfig proxy.config.cluster.mcport 5089 INT

	setAtsConfig proxy.config.admin.synthetic_port 5083 INT
	setAtsConfig proxy.config.admin.autoconf_port 5085 INT

	setAtsConfig proxy.config.process_manager.mgmt_port 5084 INT
	setAtsConfig proxy.config.url_remap.default_to_server_pac_port 5087 INT
	
	setAtsConfig proxy.config.http.auth_server_session_private 1 INT
	setAtsConfig proxy.config.http.attach_server_session_to_client 1 INT
	setAtsConfig proxy.config.http.server_session_sharing.match none STRING
	setAtsConfig proxy.config.http.connect_ports 0-65535 STRING

	# set ATS exceptions

	cat $ATS_ETC_DIR/cache.config | grep "#" > /tmp/a.txt
	cp -f /tmp/a.txt $ATS_ETC_DIR/cache.config
	rm -f /tmp/a.txt

	if [ -f $CACHE_EXCEPTION_LIST ]; then
		for line in `cat $CACHE_EXCEPTION_LIST |grep -v "#"`; do
			if [ "$line" != "" ]; then
				addATSException $line
			fi
		done
	fi

	# second, set the ats can open 65535 file descriptors, start ats
	ulimit -HSn 65535

	if  fnIsProcessAlive "traffic_server" ; then
		dbgLog "ats is already running."

		${ATS_BIN_DIR}/trafficserver restart >/dev/null 2>&1
	else
		dbgLog "start ATS service..."

		${ATS_BIN_DIR}/trafficserver start >/dev/null 2>&1
	fi

	ATS_PORT=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "ats_listen_port")

	# third, restart the iptables to apply the redirect rule
	line=`iptables -t nat -L -n |grep "ports $ATS_PORT" | grep "dpt:$DPT_PORT"`
	if [ "$line" = "" ] ; then
		dbgLog "restart the firewall..."
		service iptables restart >/dev/null 2>&1

		WCCPD_STR=`ps -ef|grep wccpd|grep -v grep`
		if [ "$WCCPD_STR" != "" ]; then
			/var/iwss/S99ISWCCPd stop >/dev/null 2>&1
			/var/iwss/S99ISWCCPd start >/dev/null 2>&1
		fi
	fi
}

fnEnableCache()
{
	dbgLog "fnEnableCache $@"

	# fisrt, check if iwss was deployed in proxy mode, if yes, then set cache as enabled.
	DeployMode=$($INICONFIG -r -f $INTSCAN_CONF -s "bridge" -o "bridgemode")
	if [ "$DeployMode" != "0" ]; then
		dbgLog "Cache can not be enabled under current deploy mode."
		exit 1
	fi

	DeployMode=$($INICONFIG -r -f $HTTP_CONF -s "http" -o "transparency")
	dbgLog "Current deploy mode is $DeployMode"

	#TT225749, modified by johnson, to support content cache in WCCP mode, 2011-08-24
	if [[ "$DeployMode" = "no" || "$DeployMode" = "wccp" ]]; then
		# fowarding proxy, wccp, forwarding proxy with upstream proxy
		dbgLog DeployMode is proxy or wccp
	else
		dbgLog "Cache can not be enabled under current deploy mode."
		exit 1
	fi

	DeployMode=$($INICONFIG -r -f $HTTP_CONF -s "http" -o "self_proxy")
	if [[ "$DeployMode" = "yes" || "$DeployMode" = "no" ]]; then
		# not reverse proxy mode
		dbgLog DeployMode is not reverse proxy
	else
		dbgLog "Cache can not be enabled under current deploy mode."
		exit 1
	fi

	# set cache enabled
	$INICONFIG -w -f $CACHE_CONF -s "$CACHE_SECTION" -o "enable" -n "yes"

	# prepare ATS binaries
	if [ ! -f "$ATS_BIN_DIR/trafficserver" ]; then
		mkdir -p $ATS_ROOT_DIR
		cp -fv /var/monitor/ats.tar.gz $ATS_ROOT_DIR
		(cd $ATS_ROOT_DIR; tar xvfz ats.tar.gz)
	fi

	if [ ! "$2" = "no" ]; then
		fnStartATS
	fi
}

fnStopATS()
{
	if [ ! -f ${ATS_BIN_DIR}/trafficserver ]; then
		return
	fi

	if  fnIsProcessAlive "traffic_server" ; then
		dbgLog "stop ATS service..."

		${ATS_BIN_DIR}/trafficserver stop >/dev/null 2>&1

		killall -9 traffic_cop
		killall -9 traffic_manager
		killall -9 traffic_server
	else
		dbgLog "ATS has already been stoped"
	fi

	getATSStorage

	CHECK_TMPFS=`mount | grep -v grep | grep $ATS_CACHE_LOCATION | wc -l`
	if [ $CHECK_TMPFS -ne 0 ]; then
		dbgLog umount $ATS_CACHE_LOCATION

		umount $ATS_CACHE_LOCATION
	fi
}

fnDisableCache()
{
	dbgLog "fnDisableCache $@"

	# first, set the cache as disabled
	$INICONFIG -w -f $CACHE_CONF -s "$CACHE_SECTION" -o "enable" -n "no"

	#second, restar the iptables to remove the redirect rule
	dbgLog "restart the firewall..."
	service iptables restart >/dev/null 2>&1

	WCCPD_STR=`ps -ef|grep wccpd|grep -v grep`
	if [ "$WCCPD_STR" != "" ]; then
		/var/iwss/S99ISWCCPd stop >/dev/null 2>&1
		/var/iwss/S99ISWCCPd start >/dev/null 2>&1
	fi

	fnStopATS

	getATSStorage

	if [ "$ATS_CACHE_LOCATION" != "" ]; then
		if [ -d "$ATS_CACHE_LOCATION" ]; then
			rm -f $ATS_CACHE_LOCATION/*
		fi
	fi
}

fnClearCache()
{
	dbgLog "fnClearCache $@"

	if [ -f $CACHE_CLEAR_MARK ]; then
		return
	fi

	echo > $CACHE_CLEAR_MARK

	${ATS_BIN_DIR}/trafficserver stop >/dev/null 2>&1

	killall -9 traffic_cop
	killall -9 traffic_manager
	killall -9 traffic_server

	${ATS_BIN_DIR}/traffic_server -Cclear  >/dev/null 2>&1
	${ATS_BIN_DIR}/trafficserver start >/dev/null 2>&1

	# third, write clear cache time
	Time=$(date -d "`date`"  +%s)
	$INICONFIG -w -f $CACHE_CONF -s "$CACHE_SECTION" -o "last_clear_date" -n "$Time"

	rm -f $CACHE_CLEAR_MARK

	return 0
}

fnGetCacheInfo()
{
	return 0
}

fnGetMaxObj()
{
	line=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "max_cached_object_size")
	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "maximumobject" -n "$line"
}

fnSetMaxObj()
{
	setAtsConfig proxy.config.cache.max_doc_size $2 INT
}

fnGetRateInfo()
{
	#get number of objects
	numobjects=
	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "numbers_objects" -n "$numobjects"

	#get reques hit in past 60min
	request_hit=
	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "request_hit" -n "$request_hit"

	#get byte hit in past 60min
	byte_hit=
	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "byte_hit" -n "$byte_hit"

	cache_disk_usage=
	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "cache_disk_usage" -n "$cache_disk_usage"
}

# $2= location, $3=size, $4=memory_cache
fnSetCacheSpace()
{
	if [ "$2" != "" -a "$2" != "null" ]; then
		$INICONFIG -w -f $CACHE_CONF -s "Cache" -o "cache_directory" -n $2
	fi

	if [ "$3" != "" -a "$3" != "null" ]; then
		$INICONFIG -w -f $CACHE_CONF -s "Cache" -o "cache_size" -n $3
	fi

	if [ "$4" = "yes" -a "$4" != "null" ]; then
		$INICONFIG -w -f $CACHE_CONF -s "Cache" -o "memory_cache" -n yes
	else
		$INICONFIG -w -f $CACHE_CONF -s "Cache" -o "memory_cache" -n no
	fi
}

fnGetMemInfo()
{
	line=`cat /proc/meminfo |grep "MemTotal" |awk -F " " '{print $2}'`
	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "mem_info" -n "$line"
	echo $line
}

fnGetDiskInfo()
{
	ATS_CACHE_LOCATION=$($INICONFIG -r -f $CACHE_CONF -s "$CACHE_SECTION" -o "cache_directory")

	line=`df -k /var`
	line=`echo $line |awk -F " " '{print $9}'`

	/usr/bin/iniconfig -w -f $CACHE_CONF -s "$RECORD_SECTION" -o "disk_info" -n "$line"

	echo $line
}

main()
{
	action="$1"

	case $action in
		EnableCache)
			fnEnableCache $@
			;;
		DisableCache)
			fnDisableCache $@
			;;
		StartATS)
			fnStartATS $@
			;;
		StopATS)
			fnStopATS $@
			;;
		ClearCache)
			fnClearCache $@
			;;
		GetCacheInfo)
			fnGetCacheInfo $@
			;;
		GetMaxObj)
			fnGetMaxObj $@
			;;
		SetMaxObj)
			fnSetMaxObj $@
			;;
		GetRateInfo)
			fnGetRateInfo $@
			;;
		SetCacheSpace)
			fnSetCacheSpace $@
			;;
		GetMemInfo)
			fnGetMemInfo $@
			;;
		GetDiskInfo)
			fnGetDiskInfo $@
			;;
		*)
			dbgLog "Unrecgonized action: $action"
			exit 1
			;;
	esac
}

if [ "$1" != "" ]; then
	main $@
	exit $?
fi

