#coding: utf-8

#from common.commonId import logformat
from agent_config   import AgentConfig

from logFiltering import ILogFiltering
import logging


class LogFilteringByHits(ILogFiltering):

    """Filtering normalized access log blocks by hits.

    By default, the input is 1 minutes logs by default.
    Return filtered log blocks.

    """
    #logformat = AgentConfig.IWSS_ACCESS_SCHEMA_ITEMS
    logformat = AgentConfig.UNIFIED_SCHEMA_ITEMS
    # Index list
    idxURL = logformat.index('CLF_URL')
    idxUserID = logformat.index('CLF_UserName')
    #idxAction = #logformat.index('operation')  # CLF_Action')
    idxDomain = logformat.index('CLF_Domain') #CLF_Domain')
    idxProtocol = logformat.index('CLF_Channel') #CLF_Channel')
    idxFileName = logformat.index('CLF_FileName') #CLF_FileName')
    idxReferrer = logformat.index('CLF_Referrer')#CLF_Referrer')
    idxXRequestedWith = logformat.index('CLF_XRequestedWith')
    idxContentType = logformat.index('CLF_MimeType') #CLF_ContentType')
    idxMimeType = logformat.index('CLF_MimeType')#CLF_MimeType')
    idxInterruptedReason = logformat.index('CLF_InterruptedReason')#CLF_InterruptedReason')

    # Index list size related
    #idxSessionPackets = logformat.index('')#CLF_SessionPackets')
    #idxSessionBytes = logformat.index('CLF_SessionBytes')
    #idxInByte = logformat.index('CLF_InByte')
    #idxOutByte = logformat.index('CLF_OutByte')
    idxSize = logformat.index('CLF_Size') #CLF_Size')

    # Protocol list
    strHTTPProtocol = 'http://'
    strHTTPSProtocol = 'https://'
    lenHTTPProtocol = len(strHTTPProtocol)
    lenHTTPSProtocol = len(strHTTPSProtocol)

    # Resource content-type list
    # Full list reference: http://www.freeformatter.com/mime-types-list.html
    resContentTypeSet = set([
        # javascript
        'application/javascript', 'application/x-javascript',
        'text/javascript',
        # css
        'text/css',
        # image
        'image/png', 'image/gif', 'image/jpeg', 'image/webp', 'image/x-icon',
        # json
        'application/json',
        # font
        'application/font-woff', 'application/x-font-woff',
        # flash
        'application/x-shockwave-flash',
    ])

    def __init__(self):
        """Configuration by company.

        Confuration format: {
            cid1: { 'enable': True },
            cid2: { 'enable': True },
            ...
        }

        """
        self.conf = {}

    def isSameOrigin(self, domain, referrer):
        if referrer.startswith(self.strHTTPProtocol):
            return referrer.startswith(domain, self.lenHTTPProtocol)
        elif referrer.startswith(self.strHTTPSProtocol):
            return referrer.startswith(domain, self.lenHTTPSProtocol)
        return True

    def isResourceDocument(self, contentType):
        contentType = contentType.split(';')[0]
        if contentType.lower() in self.resContentTypeSet:
            return True
        return False

    def isActionAllow(self, action):
        if action in ['-', 'ALLOW']:
            return True
        return False

    def urlSplit(self, url):
        protocol, other = url.split('://', 1)
        if '/' in other:
            host, path = other.split('/', 1)
        else:
            host, path = other, ''
        if ':' in host:
            domain, port = host.split(':', 1)
        else:
            domain = host
            protocol = protocol.upper()
            if protocol == 'HTTP':
                port = 80
            elif protocol == 'HTTPS':
                port = 443
        return protocol, domain, port, '/' + path

    def logMapKey(self, log):
        return self._logMapKey(log[self.idxUserID], log[self.idxURL])

    def _logMapKey(self, uid, url):
        return '%s@%s' % (uid, url)

    def buildLogMap(self, logs):
        return {
            self.logMapKey(log): {
                'log': log, 'filtered': False
            } for log in logs
        }

    def getParentLogEntry(self, log, logMap):
        referrer = log[self.idxReferrer]
        referrerKey = self._logMapKey(log[self.idxUserID], referrer)
        if referrerKey in logMap:
            return logMap[referrerKey]
        else:
            return None

    def findParentNotFiltered(self, log, logMap):
        mapEntry = None
        referrer = log[self.idxReferrer]
        referrerKey = self._logMapKey(log[self.idxUserID], referrer)
        if referrerKey in logMap:
            mapEntry = logMap[referrerKey]
            # If the log is filtered, then traverse the parent
            refererLits = []
            while mapEntry['filtered']:
                referrer = mapEntry['log'][self.idxReferrer]
                referrerKey = self._logMapKey(
                    mapEntry['log'][self.idxUserID], referrer
                )
                if referrerKey not in logMap:
                    break
                if id(mapEntry) == id(logMap[referrerKey]):
                    break
                else:
                    if referrerKey in refererLits:
                        break
                    refererLits.append(referrerKey)
                    mapEntry = logMap[referrerKey]
        if mapEntry:
            return mapEntry['log']
        return None

    def accumulateLogFieldSize(self, srcLog, dstLog, index):
        try:
            sSize = int(srcLog[index])
        except ValueError:
            sSize = 0
        try:
            dSize = int(dstLog[index])
        except ValueError:
            dSize = 0
        except KeyError:
            raise
        dSize += sSize
        if dSize == 0:
            dstLog[index] = '0'
        else:
            dstLog[index] = str(dSize)

    def accumulateLogFieldsSize(self, srcLog, dstLog):
        # Accumulate payload includes following fields:
        # 'CLF_SessionPackets', 'CLF_SessionBytes',
        # 'CLF_InByte', 'CLF_OutByte', 'CLF_Size'
        #for index in [
        #    self.idxSessionPackets, self.idxSessionBytes,
        #    self.idxInByte, self.idxOutByte, self.idxSize
        #]:
        self.accumulateLogFieldSize(srcLog, dstLog, self.idxSize)

    def updateLogFieldsWithReferrer(self, log):
        referrer = log[self.idxReferrer]
        protocol, domain, port, path = self.urlSplit(referrer)
        log[self.idxProtocol] = protocol.upper()
        log[self.idxDomain] = domain
        log[self.idxURL] = referrer
        log[self.idxReferrer] = '-'

        fileName = path.rsplit('/', 1)[-1]
        if fileName == '':
            fileName = '-'
        log[self.idxFileName] = fileName
        log[self.idxContentType] = 'text/html'
        log[self.idxMimeType] = 'text/html'

    def accumulateLogPayload(self, log, logMap):
        # 1. Mark this log as filtered
        logMap[self.logMapKey(log)]['filtered'] = True
        parentLog = self.findParentNotFiltered(log, logMap)
        if parentLog:
            # 2. Accumulate payload related size to parent
            #logging.info(">>>>> accumulateLogFieldsSize start...")
            self.accumulateLogFieldsSize(log, parentLog)
            return None
        else:
            # 2.1. Copy current log with referrer information
            # 2.2. Add new log map entry
            newLog = log[:]
            #logging.info(">>>>> updateLogFieldsWithReferrer start")
            self.updateLogFieldsWithReferrer(newLog)
            #logging.info(">>>>> updateLogFieldsWithReferrer end")
            logMap[self.logMapKey(newLog)] = {
                'log': newLog, 'filtered': False
            }
            return newLog

    def setLogFilteringConfiguration(self, conf):
        # TODO: format check with validictory
        if not isinstance(conf, dict):
            return False
        self.conf.update(conf)
        return True

    def isLogFilteringEnabled(self, cid):
        if cid is None:
            return False
        config = self.conf.get(cid)
        if config and config['enable']:
            return True
        return False

    def doLogFiltering(self, logs, cid=None):
        """Filtering by hits.

        1. Build log map for summary
            Key:    {{protocol}}://{{domain}}{{path}}
            Value:  {
                        log: {{log_entry}},
                        filtered: True/False,
                        parent: {{log_entry}}
                    }
        2. Traverse the logs one by one with filtering logic
        """
        #if not self.isLogFilteringEnabled(cid):
        #    return logs
        result = []
        logMap = self.buildLogMap(logs)
        for log in logs:
            #action = log[self.idxAction]
            # 0. Only filter log with action ALLOW or empty
            #if not self.isActionAllow(action):
            #    result.append(log)
            #    continue
            referrer = log[self.idxReferrer]
            #logging.info(">>>> log:%r,  referrer:%r", log, referrer)
            # 1. Do not filter is no referrer present
            if not referrer or referrer == '-':
                result.append(log)
                continue
            #for SEGTT 337212 since some rerferrers do not start with http or https
            if  not referrer.startswith(self.strHTTPProtocol) and not referrer.startswith(self.strHTTPSProtocol):
                result.append(log)
                continue  
            domain = log[self.idxDomain]
            # 2. Check referrer and host whether they are same domain or not
            if self.isSameOrigin(domain, referrer):
                # 2.1 Same origin, accumulate payload to parent
                logging.debug("domain and refer are same")
                newLog = self.accumulateLogPayload(log, logMap)
                if newLog is not None:
                    logging.debug(" domain and refer same can not find parent log")
                    result.append(newLog)
            else:
                # 2.1 Not same origin
                # 2.1.1 Check whether ajax request
                if log[self.idxXRequestedWith].lower() == 'xmlhttprequest':
                    # ajax, accumulate payload to parent
                    logging.debug("ajax request")
                    newLog = self.accumulateLogPayload(log, logMap)
                    if newLog is not None:
                        logging.debug("ajax request but can not find parentLog")
                        result.append(newLog)
                # 2.1.2 Different origin, then check content type
                elif self.isResourceDocument(log[self.idxContentType]):
                    # External resource, accumulate payload to parent
                    logging.debug("contentType match Ok")
                    newLog = self.accumulateLogPayload(log, logMap)
                    if newLog is not None:
                        logging.debug("contentType match but can not find parentLog")
                        result.append(newLog)
                #2.1.3 Different origin, then check content type good
                else:
                    logEntry = self.getParentLogEntry(log, logMap)
                    logging.debug("not same domain and refer, not ajax, not contentType match")
                    # 2.1.3.1 Check whether parent has been filtered or not
                    if logEntry is None or not logEntry['filtered']:
                        result.append(log)
                    # 2.1.3.2 Accumulate if parent has been filtered
                    else:
                        newLog = self.accumulateLogPayload(log, logMap)
                        if newLog is not None:
                            result.append(newLog)
        return result
